@@ -3,7 +3,7 @@
  *               -> gives the feeling of a real room with a simple headphone
  *****************************************************************************
  * Copyright (C) 2002 VideoLAN
- * $Id: headphone.c 6961 2004-03-05 17:34:23Z sam $
+ * $Id: headphone.c 8845 2004-09-29 09:00:41Z zorglub $
  *
  * Authors: Boris Dors <babal@via.ecp.fr>
  *
@@ -55,8 +55,7 @@ static void DoWork    ( aout_instance_t *, aout_filter_t *, aout_buffer_t *,
 
 #define HEADPHONE_DIM_TEXT N_("Characteristic dimension")
 #define HEADPHONE_DIM_LONGTEXT N_( \
-     "Headphone virtual spatialization effect parameter: "\
-     "distance between front left speaker and listener in meters.")
+     "Distance between front left speaker and listener in meters.")
 
 vlc_module_begin();
     set_description( N_("headphone channel mixer with virtual spatialization effect") );
@@ -306,6 +305,7 @@ static int Create( vlc_object_t *p_this )
           || (p_filter->input.i_format != VLC_FOURCC('f','l','3','2')
                && p_filter->input.i_format != VLC_FOURCC('f','i','3','2')) )
     {
+        msg_Dbg( p_filter, "Filter discarded (invalid format)" );
         return -1;
     }
 
