@@ -1,8 +1,8 @@
 /*****************************************************************************
  * hotkeys.c: Hotkey handling for vlc
  *****************************************************************************
- * Copyright (C) 2003 VideoLAN
- * $Id: hotkeys.c,v 1.15 2003/12/15 13:38:27 hartman Exp $
+ * Copyright (C) 2004 VideoLAN
+ * $Id: hotkeys.c 8966 2004-10-10 10:08:44Z ipkiss $
  *
  * Authors: Sigmund Augdal <sigmunau@idi.ntnu.no>
  *
@@ -10,7 +10,7 @@
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -36,6 +36,12 @@
 #include "vlc_keys.h"
 
 #define BUFFER_SIZE 10
+
+#define CHANNELS_NUMBER 4
+#define VOLUME_TEXT_CHAN     p_intf->p_sys->p_channels[ 0 ]
+#define VOLUME_WIDGET_CHAN   p_intf->p_sys->p_channels[ 1 ]
+#define POSITION_TEXT_CHAN   p_intf->p_sys->p_channels[ 2 ]
+#define POSITION_WIDGET_CHAN p_intf->p_sys->p_channels[ 3 ]
 /*****************************************************************************
  * intf_sys_t: description and status of FB interface
  *****************************************************************************/
@@ -48,6 +54,8 @@ struct intf_sys_t
     int                 p_keys[ BUFFER_SIZE ]; /* buffer that contains
                                                 * keyevents */
     int                 i_size;        /* number of events in buffer */
+    int                 p_channels[ CHANNELS_NUMBER ]; /* contains registered
+                                                        * channel IDs */
     input_thread_t *    p_input;       /* pointer to input */
     vout_thread_t *     p_vout;        /* pointer to vout object */
 };
@@ -65,6 +73,9 @@ static int  ActionKeyCB( vlc_object_t *, char const *,
                          vlc_value_t, vlc_value_t, void * );
 static void PlayBookmark( intf_thread_t *, int );
 static void SetBookmark ( intf_thread_t *, int );
+static void DisplayPosition( intf_thread_t *, vout_thread_t *, input_thread_t * );
+static void DisplayVolume  ( intf_thread_t *, vout_thread_t *, audio_volume_t );
+static void ClearChannels  ( intf_thread_t *, vout_thread_t * );
 
 /*****************************************************************************
  * Module descriptor
@@ -80,31 +91,31 @@ static void SetBookmark ( intf_thread_t *, int );
 #define BOOKMARK9_TEXT N_("Playlist bookmark 9")
 #define BOOKMARK10_TEXT N_("Playlist bookmark 10")
 #define BOOKMARK_LONGTEXT N_( \
-    "This option allows you to define playlist bookmarks")
+    "This option allows you to define playlist bookmarks.")
 
 vlc_module_begin();
-    set_description( _("hotkey interface") );
+    set_description( _("Hotkeys management interface") );
     add_string( "bookmark1", NULL, NULL,
-                BOOKMARK1_TEXT, BOOKMARK_LONGTEXT, VLC_FALSE ); 
+                BOOKMARK1_TEXT, BOOKMARK_LONGTEXT, VLC_FALSE );
     add_string( "bookmark2", NULL, NULL,
-                BOOKMARK2_TEXT, BOOKMARK_LONGTEXT, VLC_FALSE ); 
+                BOOKMARK2_TEXT, BOOKMARK_LONGTEXT, VLC_FALSE );
     add_string( "bookmark3", NULL, NULL,
-                BOOKMARK3_TEXT, BOOKMARK_LONGTEXT, VLC_FALSE ); 
+                BOOKMARK3_TEXT, BOOKMARK_LONGTEXT, VLC_FALSE );
     add_string( "bookmark4", NULL, NULL,
-                BOOKMARK4_TEXT, BOOKMARK_LONGTEXT, VLC_FALSE ); 
+                BOOKMARK4_TEXT, BOOKMARK_LONGTEXT, VLC_FALSE );
     add_string( "bookmark5", NULL, NULL,
-                BOOKMARK5_TEXT, BOOKMARK_LONGTEXT, VLC_FALSE ); 
+                BOOKMARK5_TEXT, BOOKMARK_LONGTEXT, VLC_FALSE );
     add_string( "bookmark6", NULL, NULL,
-                BOOKMARK6_TEXT, BOOKMARK_LONGTEXT, VLC_FALSE ); 
+                BOOKMARK6_TEXT, BOOKMARK_LONGTEXT, VLC_FALSE );
     add_string( "bookmark7", NULL, NULL,
-                BOOKMARK7_TEXT, BOOKMARK_LONGTEXT, VLC_FALSE ); 
+                BOOKMARK7_TEXT, BOOKMARK_LONGTEXT, VLC_FALSE );
     add_string( "bookmark8", NULL, NULL,
-                BOOKMARK8_TEXT, BOOKMARK_LONGTEXT, VLC_FALSE ); 
+                BOOKMARK8_TEXT, BOOKMARK_LONGTEXT, VLC_FALSE );
     add_string( "bookmark9", NULL, NULL,
-                BOOKMARK9_TEXT, BOOKMARK_LONGTEXT, VLC_FALSE ); 
+                BOOKMARK9_TEXT, BOOKMARK_LONGTEXT, VLC_FALSE );
     add_string( "bookmark10", NULL, NULL,
-                BOOKMARK10_TEXT, BOOKMARK_LONGTEXT, VLC_FALSE ); 
-    
+                BOOKMARK10_TEXT, BOOKMARK_LONGTEXT, VLC_FALSE );
+
     set_capability( "interface", 0 );
     set_callbacks( Open, Close );
 vlc_module_end();
@@ -161,6 +172,7 @@ static void Run( intf_thread_t *p_intf )
     playlist_t *p_playlist;
     input_thread_t *p_input;
     vout_thread_t *p_vout = NULL;
+    vout_thread_t *p_last_vout;
     struct hotkey *p_hotkeys = p_intf->p_vlc->p_hotkeys;
     vlc_value_t val;
     int i;
@@ -198,10 +210,10 @@ static void Run( intf_thread_t *p_intf )
         p_input = p_intf->p_sys->p_input;
 
         /* Update the vout */
+        p_last_vout = p_intf->p_sys->p_vout;
         if( p_vout == NULL )
         {
-            p_vout = vlc_object_find( p_intf, VLC_OBJECT_VOUT,
-                                      FIND_ANYWHERE );
+            p_vout = vlc_object_find( p_intf, VLC_OBJECT_VOUT, FIND_ANYWHERE );
             p_intf->p_sys->p_vout = p_vout;
         }
         else if( p_vout->b_die )
@@ -211,6 +223,16 @@ static void Run( intf_thread_t *p_intf )
             p_intf->p_sys->p_vout = NULL;
         }
 
+        /* Register OSD channels */
+        if( p_vout && p_vout != p_last_vout )
+        {
+            for( i = 0; i < CHANNELS_NUMBER; i++ )
+            {
+                spu_Control( p_vout->p_spu, SPU_CHANNEL_REGISTER,
+                             &p_intf->p_sys->p_channels[ i ] );
+            }
+        }
+
         /* Find action triggered by hotkey */
         i_action = 0;
         i_key = GetKey( p_intf );
@@ -232,45 +254,90 @@ static void Run( intf_thread_t *p_intf )
         if( i_action == ACTIONID_QUIT )
         {
             p_intf->p_vlc->b_die = VLC_TRUE;
-            vout_OSDMessage( VLC_OBJECT(p_intf), _("Quit" ) );
+            ClearChannels( p_intf, p_vout );
+            vout_OSDMessage( p_intf, DEFAULT_CHAN, _( "Quit" ) );
             continue;
         }
         else if( i_action == ACTIONID_VOL_UP )
         {
             audio_volume_t i_newvol;
-            char string[9];
             aout_VolumeUp( p_intf, 1, &i_newvol );
-            sprintf( string, "Vol %d%%", i_newvol*100/AOUT_VOLUME_MAX );
-            vout_OSDMessage( VLC_OBJECT(p_intf), string );
+            DisplayVolume( p_intf, p_vout, i_newvol );
         }
         else if( i_action == ACTIONID_VOL_DOWN )
         {
             audio_volume_t i_newvol;
-            char string[9];
             aout_VolumeDown( p_intf, 1, &i_newvol );
-            sprintf( string, "Vol %d%%", i_newvol*100/AOUT_VOLUME_MAX );
-            vout_OSDMessage( VLC_OBJECT(p_intf), string );
+            DisplayVolume( p_intf, p_vout, i_newvol );
         }
         else if( i_action == ACTIONID_VOL_MUTE )
         {
             audio_volume_t i_newvol = -1;
             aout_VolumeMute( p_intf, &i_newvol );
-            if( i_newvol == 0 )
+            if( p_vout )
             {
-                vout_OSDMessage( VLC_OBJECT(p_intf), "Mute" );
+                if( i_newvol == 0 )
+                {
+                    ClearChannels( p_intf, p_vout );
+                    vout_OSDIcon( VLC_OBJECT( p_intf ), DEFAULT_CHAN,
+                                  OSD_MUTE_ICON );
+                }
+                else
+                {
+                    DisplayVolume( p_intf, p_vout, i_newvol );
+                }
             }
-            else
+        }
+        else if( i_action == ACTIONID_INTF_SHOW )
+        {
+            val.b_bool = VLC_TRUE;
+            p_playlist = vlc_object_find( p_intf, VLC_OBJECT_PLAYLIST,
+                                          FIND_ANYWHERE );
+            if( p_playlist )
             {
-                char string[9];
-                sprintf( string, "Vol %d%%", i_newvol*100/AOUT_VOLUME_MAX );
-                vout_OSDMessage( VLC_OBJECT(p_intf), string );
+                var_Set( p_playlist, "intf-show", val );
+                vlc_object_release( p_playlist );
             }
         }
+        else if( i_action == ACTIONID_SUBDELAY_DOWN )
+        {
+            int64_t i_delay = var_GetTime( p_input, "spu-delay" );
+
+            i_delay -= 50000;    /* 50 ms */
+
+            var_SetTime( p_input, "spu-delay", i_delay );
+            ClearChannels( p_intf, p_vout );
+            vout_OSDMessage( p_intf, DEFAULT_CHAN, "Subtitle delay %i ms",
+                                 (int)(i_delay/1000) );
+        }
+        else if( i_action == ACTIONID_SUBDELAY_UP )
+        {
+            int64_t i_delay = var_GetTime( p_input, "spu-delay" );
+
+            i_delay += 50000;    /* 50 ms */
+
+            var_SetTime( p_input, "spu-delay", i_delay );
+            ClearChannels( p_intf, p_vout );
+            vout_OSDMessage( p_intf, DEFAULT_CHAN, "Subtitle delay %i ms",
+                                 (int)(i_delay/1000) );
+        }
         else if( i_action == ACTIONID_FULLSCREEN )
         {
             if( p_vout )
             {
-                p_vout->i_changes |= VOUT_FULLSCREEN_CHANGE;
+                var_Get( p_vout, "fullscreen", &val ); val.b_bool = !val.b_bool;
+                var_Set( p_vout, "fullscreen", val );
+            }
+            else
+            {
+                p_playlist = vlc_object_find( p_intf, VLC_OBJECT_PLAYLIST,
+                                          FIND_ANYWHERE );
+                if( p_playlist )
+                {
+                    var_Get( p_playlist, "fullscreen", &val ); val.b_bool = !val.b_bool;
+                    var_Set( p_playlist, "fullscreen", val );
+                    vlc_object_release( p_playlist );
+                }
             }
         }
         else if( i_action == ACTIONID_PLAY )
@@ -279,13 +346,11 @@ static void Run( intf_thread_t *p_intf )
                                           FIND_ANYWHERE );
             if( p_playlist )
             {
-                vlc_mutex_lock( &p_playlist->object_lock );
-                if( p_playlist->i_size )
-                {
-                    vlc_mutex_unlock( &p_playlist->object_lock );
-                    playlist_Play( p_playlist );
-                    vlc_object_release( p_playlist );
-                }
+                ClearChannels( p_intf, p_vout );
+                vout_OSDIcon( VLC_OBJECT( p_intf ), DEFAULT_CHAN,
+                              OSD_PAUSE_ICON );
+                playlist_Play( p_playlist );
+                vlc_object_release( p_playlist );
             }
         }
         else if( i_action == ACTIONID_PLAY_PAUSE )
@@ -297,7 +362,9 @@ static void Run( intf_thread_t *p_intf )
             }
             if( p_input && val.i_int != PAUSE_S )
             {
-                vout_OSDMessage( VLC_OBJECT(p_intf), _( "Pause" ) );
+                ClearChannels( p_intf, p_vout );
+                vout_OSDIcon( VLC_OBJECT( p_intf ), DEFAULT_CHAN,
+                              OSD_PAUSE_ICON );
                 val.i_int = PAUSE_S;
                 var_Set( p_input, "state", val );
             }
@@ -307,60 +374,151 @@ static void Run( intf_thread_t *p_intf )
                                               FIND_ANYWHERE );
                 if( p_playlist )
                 {
-                    vlc_mutex_lock( &p_playlist->object_lock );
-                    if( p_playlist->i_size )
-                    {
-                        vlc_mutex_unlock( &p_playlist->object_lock );
-                        vout_OSDMessage( VLC_OBJECT(p_intf), _( "Play" ) );
-                        playlist_Play( p_playlist );
-                        vlc_object_release( p_playlist );
-                    }
+                    ClearChannels( p_intf, p_vout );
+                    vout_OSDIcon( VLC_OBJECT( p_intf ), DEFAULT_CHAN,
+                                  OSD_PLAY_ICON );
+                    playlist_Play( p_playlist );
+                    vlc_object_release( p_playlist );
                 }
             }
         }
         else if( p_input )
         {
+            /* FIXME --fenrir
+             * How to get a valid value ?
+             * That's not that easy with some special stream
+             */
+            vlc_bool_t b_seekable = VLC_TRUE;
+
             if( i_action == ACTIONID_PAUSE )
             {
-                vout_OSDMessage( VLC_OBJECT(p_intf), _( "Pause" ) );
+                ClearChannels( p_intf, p_vout );
+                vout_OSDIcon( VLC_OBJECT( p_intf ), DEFAULT_CHAN,
+                              OSD_PAUSE_ICON );
                 val.i_int = PAUSE_S;
                 var_Set( p_input, "state", val );
             }
-            else if( i_action == ACTIONID_JUMP_BACKWARD_10SEC )
+            else if( i_action == ACTIONID_JUMP_BACKWARD_10SEC && b_seekable )
             {
-                vout_OSDMessage( VLC_OBJECT(p_intf), _( "Jump -10 seconds" ) );
                 val.i_time = -10000000;
                 var_Set( p_input, "time-offset", val );
+                DisplayPosition( p_intf, p_vout, p_input );
             }
-            else if( i_action == ACTIONID_JUMP_FORWARD_10SEC )
+            else if( i_action == ACTIONID_JUMP_FORWARD_10SEC && b_seekable )
             {
-                vout_OSDMessage( VLC_OBJECT(p_intf), _( "Jump +10 seconds" ) );
                 val.i_time = 10000000;
                 var_Set( p_input, "time-offset", val );
+                DisplayPosition( p_intf, p_vout, p_input );
             }
-            else if( i_action == ACTIONID_JUMP_BACKWARD_1MIN )
+            else if( i_action == ACTIONID_JUMP_BACKWARD_1MIN && b_seekable )
             {
-                vout_OSDMessage( VLC_OBJECT(p_intf), _( "Jump -1 minute" ) );
                 val.i_time = -60000000;
                 var_Set( p_input, "time-offset", val );
+                DisplayPosition( p_intf, p_vout, p_input );
             }
-            else if( i_action == ACTIONID_JUMP_FORWARD_1MIN )
+            else if( i_action == ACTIONID_JUMP_FORWARD_1MIN && b_seekable )
             {
-                vout_OSDMessage( VLC_OBJECT(p_intf), _( "Jump +1 minute" ) );
                 val.i_time = 60000000;
                 var_Set( p_input, "time-offset", val );
+                DisplayPosition( p_intf, p_vout, p_input );
             }
-            else if( i_action == ACTIONID_JUMP_BACKWARD_5MIN )
+            else if( i_action == ACTIONID_JUMP_BACKWARD_5MIN && b_seekable )
             {
-                vout_OSDMessage( VLC_OBJECT(p_intf), _( "Jump -5 minutes" ) );
                 val.i_time = -300000000;
                 var_Set( p_input, "time-offset", val );
+                DisplayPosition( p_intf, p_vout, p_input );
             }
-            else if( i_action == ACTIONID_JUMP_FORWARD_5MIN )
+            else if( i_action == ACTIONID_JUMP_FORWARD_5MIN && b_seekable )
             {
-                vout_OSDMessage( VLC_OBJECT(p_intf), _( "Jump +5 minutes" ) );
                 val.i_time = 300000000;
                 var_Set( p_input, "time-offset", val );
+                DisplayPosition( p_intf, p_vout, p_input );
+            }
+            else if( i_action == ACTIONID_AUDIO_TRACK )
+            {
+                vlc_value_t val, list, list2;
+                int i_count, i;
+                var_Get( p_input, "audio-es", &val );
+                var_Change( p_input, "audio-es", VLC_VAR_GETCHOICES,
+                            &list, &list2 );
+                i_count = list.p_list->i_count;
+                if( i_count <= 1 )
+                {
+                    continue;
+                }
+                for( i = 0; i < i_count; i++ )
+                {
+                    if( val.i_int == list.p_list->p_values[i].i_int )
+                    {
+                        break;
+                    }
+                }
+                /* value of audio-es was not in choices list */
+                if( i == i_count )
+                {
+                    msg_Warn( p_input,
+                              "invalid current audio track, selecting 0" );
+                    var_Set( p_input, "audio-es",
+                             list.p_list->p_values[0] );
+                    i = 0;
+                }
+                else if( i == i_count - 1 )
+                {
+                    var_Set( p_input, "audio-es",
+                             list.p_list->p_values[1] );
+                    i = 1;
+                }
+                else
+                {
+                    var_Set( p_input, "audio-es",
+                             list.p_list->p_values[i+1] );
+                    i++;
+                }
+                vout_OSDMessage( VLC_OBJECT(p_input), DEFAULT_CHAN,
+                                 _("Audio track: %s"),
+                                 list2.p_list->p_values[i].psz_string );
+            }
+            else if( i_action == ACTIONID_SUBTITLE_TRACK )
+            {
+                vlc_value_t val, list, list2;
+                int i_count, i;
+                var_Get( p_input, "spu-es", &val );
+                
+                var_Change( p_input, "spu-es", VLC_VAR_GETCHOICES,
+                            &list, &list2 );
+                i_count = list.p_list->i_count;
+                if( i_count <= 1 )
+                {
+                    vout_OSDMessage( VLC_OBJECT(p_input), DEFAULT_CHAN, _("Subtitle track: %s"), _("N/A") );
+                    continue;
+                }
+                for( i = 0; i < i_count; i++ )
+                {
+                    if( val.i_int == list.p_list->p_values[i].i_int )
+                    {
+                        break;
+                    }
+                }
+                /* value of spu-es was not in choices list */
+                if( i == i_count )
+                {
+                    msg_Warn( p_input, "invalid current subtitle track, selecting 0" );
+                    var_Set( p_input, "spu-es", list.p_list->p_values[0] );
+                    i = 0;
+                }
+                else if( i == i_count - 1 )
+                {
+                    var_Set( p_input, "spu-es", list.p_list->p_values[0] );
+                    i = 0;
+                }
+                else
+                {
+                    var_Set( p_input, "spu-es", list.p_list->p_values[i+1] );
+                    i = i + 1;
+                }
+                vout_OSDMessage( VLC_OBJECT(p_input), DEFAULT_CHAN,
+                                 _("Subtitle track: %s"),
+                                 list2.p_list->p_values[i].psz_string );
             }
             else if( i_action == ACTIONID_NEXT )
             {
@@ -368,6 +526,7 @@ static void Run( intf_thread_t *p_intf )
                                               FIND_ANYWHERE );
                 if( p_playlist )
                 {
+                    vout_OSDMessage( VLC_OBJECT(p_input), DEFAULT_CHAN, _("Next") );
                     playlist_Next( p_playlist );
                     vlc_object_release( p_playlist );
                 }
@@ -378,6 +537,7 @@ static void Run( intf_thread_t *p_intf )
                                               FIND_ANYWHERE );
                 if( p_playlist )
                 {
+                    vout_OSDMessage( VLC_OBJECT(p_input), DEFAULT_CHAN, _("Previous") );
                     playlist_Prev( p_playlist );
                     vlc_object_release( p_playlist );
                 }
@@ -396,46 +556,17 @@ static void Run( intf_thread_t *p_intf )
             {
                 vlc_value_t val; val.b_bool = VLC_TRUE;
                 var_Set( p_input, "rate-faster", val );
+                vout_OSDMessage( VLC_OBJECT(p_input), DEFAULT_CHAN, _("Faster") );
             }
-            else if( i_action == ACTIONID_FASTER )
+            else if( i_action == ACTIONID_SLOWER )
             {
                 vlc_value_t val; val.b_bool = VLC_TRUE;
                 var_Set( p_input, "rate-slower", val );
+                vout_OSDMessage( VLC_OBJECT(p_input), DEFAULT_CHAN, _("Slower") );
             }
-            else if( i_action == ACTIONID_POSITION )
+            else if( i_action == ACTIONID_POSITION && b_seekable )
             {
-                playlist_t *p_playlist = vlc_object_find( p_intf, VLC_OBJECT_PLAYLIST,
-                                              FIND_ANYWHERE );
-                char psz_duration[MSTRTIME_MAX_SIZE];
-                char psz_time[MSTRTIME_MAX_SIZE];
-                vlc_value_t time;
-                mtime_t i_seconds;
-                                                                                                                            
-                var_Get( p_input, "time", &time );
-                                                                                                                            
-                if( p_playlist )
-                {
-                    mtime_t dur =
-                        p_playlist->pp_items[p_playlist->i_index]->i_duration;
-
-                    i_seconds = time.i_time / 1000000;
-                    secstotimestr ( psz_time, i_seconds );
-                                                                                                                            
-                    if( dur != -1 )
-                    {
-                        char psz_position[2*MSTRTIME_MAX_SIZE + 3];
-                        secstotimestr( psz_duration, dur/1000000 );
-                        strcpy( psz_position, psz_time );
-                        strcat( psz_position, " / " );
-                        strcat( psz_position, psz_duration );
-                        vout_OSDMessage( VLC_OBJECT(p_playlist), psz_position );
-                    }
-                    else if( i_seconds > 0 )
-                    { 
-                        vout_OSDMessage( VLC_OBJECT(p_playlist), psz_time );
-                    }
-                    vlc_object_release( p_playlist );
-                }
+                DisplayPosition( p_intf, p_vout, p_input );
             }
             else if( i_action >= ACTIONID_PLAY_BOOKMARK1 &&
                      i_action <= ACTIONID_PLAY_BOOKMARK10 )
@@ -483,7 +614,7 @@ static int KeyEvent( vlc_object_t *p_this, char const *psz_var,
     vlc_mutex_lock( &p_intf->p_sys->change_lock );
     if ( p_intf->p_sys->i_size == BUFFER_SIZE )
     {
-        msg_Warn( p_intf, "Event buffer full, dropping keypress" );
+        msg_Warn( p_intf, "event buffer full, dropping keypress" );
         vlc_mutex_unlock( &p_intf->p_sys->change_lock );
         return VLC_EGENERIC;
     }
@@ -516,23 +647,24 @@ static int ActionKeyCB( vlc_object_t *p_this, char const *psz_var,
 }
 
 static void PlayBookmark( intf_thread_t *p_intf, int i_num )
-{   
+{
     vlc_value_t val;
     int i_position;
     char psz_bookmark_name[11];
-    playlist_t *p_playlist = vlc_object_find( p_intf,
-             VLC_OBJECT_PLAYLIST, FIND_ANYWHERE );
-    
+    playlist_t *p_playlist =
+        vlc_object_find( p_intf, VLC_OBJECT_PLAYLIST, FIND_ANYWHERE );
+
     sprintf( psz_bookmark_name, "bookmark%i", i_num );
     var_Create( p_intf, psz_bookmark_name, VLC_VAR_STRING|VLC_VAR_DOINHERIT );
     var_Get( p_intf, psz_bookmark_name, &val );
-                                                                                                                    
+
     if( p_playlist )
     {
         char *psz_bookmark = strdup( val.psz_string );
-        for( i_position = 0 ; i_position < p_playlist->i_size ; i_position++)
+        for( i_position = 0; i_position < p_playlist->i_size; i_position++)
         {
-            if( !strcmp( psz_bookmark, p_playlist->pp_items[i_position]->psz_uri ) )
+            if( !strcmp( psz_bookmark,
+                         p_playlist->pp_items[i_position]->input.psz_uri ) )
             {
                 playlist_Goto( p_playlist, i_position );
                 break;
@@ -545,16 +677,92 @@ static void PlayBookmark( intf_thread_t *p_intf, int i_num )
 static void SetBookmark( intf_thread_t *p_intf, int i_num )
 {
     vlc_value_t val;
-    playlist_t *p_playlist = vlc_object_find( p_intf,
-            VLC_OBJECT_PLAYLIST, FIND_ANYWHERE );
+    playlist_t *p_playlist =
+        vlc_object_find( p_intf, VLC_OBJECT_PLAYLIST, FIND_ANYWHERE );
     if( p_playlist )
     {
         char psz_bookmark_name[11];
         sprintf( psz_bookmark_name, "bookmark%i", i_num );
-        var_Create( p_intf, psz_bookmark_name, VLC_VAR_STRING|VLC_VAR_DOINHERIT );
-        val.psz_string = strdup( p_playlist->pp_items[p_playlist->i_index]->psz_uri );
+        var_Create( p_intf, psz_bookmark_name,
+                    VLC_VAR_STRING|VLC_VAR_DOINHERIT );
+        val.psz_string = strdup( p_playlist->pp_items[p_playlist->i_index]->input.psz_uri );
         var_Set( p_intf, psz_bookmark_name, val );
-        msg_Info( p_intf, "Setting playlist bookmark %i to %s", i_num, val.psz_string );
+        msg_Info( p_intf, "setting playlist bookmark %i to %s", i_num,
+                  val.psz_string );
         vlc_object_release( p_playlist );
     }
 }
+
+static void DisplayPosition( intf_thread_t *p_intf, vout_thread_t *p_vout,
+                             input_thread_t *p_input )
+{
+    char psz_duration[MSTRTIME_MAX_SIZE];
+    char psz_time[MSTRTIME_MAX_SIZE];
+    vlc_value_t time, pos;
+    mtime_t i_seconds;
+
+    if( p_vout == NULL )
+    {
+        return;
+    }
+    ClearChannels( p_intf, p_vout );
+
+    var_Get( p_input, "time", &time );
+    i_seconds = time.i_time / 1000000;
+    secstotimestr ( psz_time, i_seconds );
+
+    var_Get( p_input, "length", &time );
+    if( time.i_time > 0 )
+    {
+        secstotimestr( psz_duration, time.i_time / 1000000 );
+        vout_OSDMessage( p_input, POSITION_TEXT_CHAN, "%s / %s",
+                         psz_time, psz_duration );
+    }
+    else if( i_seconds > 0 )
+    {
+        vout_OSDMessage( p_input, POSITION_TEXT_CHAN, psz_time );
+    }
+
+    if( !p_vout->p_parent_intf || p_vout->b_fullscreen )
+    {
+        var_Get( p_input, "position", &pos );
+        vout_OSDSlider( VLC_OBJECT( p_input ), POSITION_WIDGET_CHAN,
+                        pos.f_float * 100, OSD_HOR_SLIDER );
+    }
+}
+
+static void DisplayVolume( intf_thread_t *p_intf, vout_thread_t *p_vout,
+                           audio_volume_t i_vol )
+{
+    if( p_vout == NULL )
+    {
+        return;
+    }
+    ClearChannels( p_intf, p_vout );
+
+    if( !p_vout->p_parent_intf || p_vout->b_fullscreen )
+    {
+        vout_OSDSlider( VLC_OBJECT( p_vout ), VOLUME_WIDGET_CHAN,
+            i_vol*100/AOUT_VOLUME_MAX, OSD_VERT_SLIDER );
+    }
+    else
+    {
+        vout_OSDMessage( p_vout, VOLUME_TEXT_CHAN, "Vol %d%%",
+                         2*i_vol*100/AOUT_VOLUME_MAX );
+    }
+}
+
+static void ClearChannels( intf_thread_t *p_intf, vout_thread_t *p_vout )
+{
+    int i;
+
+    if( p_vout )
+    {
+        spu_Control( p_vout->p_spu, SPU_CHANNEL_CLEAR, DEFAULT_CHAN );
+        for( i = 0; i < CHANNELS_NUMBER; i++ )
+        {
+            spu_Control( p_vout->p_spu, SPU_CHANNEL_CLEAR,
+                         p_intf->p_sys->p_channels[ i ] );
+        }
+    }
+}
